@@ -13,7 +13,6 @@ from typing import (
     Mapping,
     Optional,
     Sequence,
-    Set,
     Tuple,
     Type,
     TypeVar,
@@ -151,14 +150,6 @@ class Smart(Generic[T]):
         if callable(smart_init := getattr(self._callable, '__smart_init__', None)):
             smart_init(self)
 
-    @property
-    def type(self) -> Type[T]:
-        return typeutil.get_return_type(self._callable)
-
-    @property
-    def dict(self) -> Dict[str, Any]:
-        return self._params.copy()
-
     def __call__(
         self,
         *args: Any,
@@ -174,8 +165,7 @@ class Smart(Generic[T]):
             An class instance.
 
         """
-        params = self.dict
-        params.pop('', None)
+        params = copy.deepcopy({k: v for k, v in self._params.items() if k})
 
         if self.check_overrides:
             typeutil.check_overrides(
@@ -224,6 +214,28 @@ class Smart(Generic[T]):
     def __hash__(self) -> int:
         return hash(strutil.join_keys(self._trace, repr(self)))
 
+    def type(
+        self,
+        return_type: bool = True,
+    ) -> Union[Type[T], Callable[..., T]]:
+        if return_type:
+            return typeutil.get_return_type(self._callable)
+        return self._callable
+
+    def dict(
+        self,
+        squash: bool = False,
+        minimal: bool = False,
+    ) -> Dict[str, Any]:
+        if squash or minimal:
+            dictionary = dict(self.items(flatten=True))
+            if minimal:
+                dictionary = dictutil.flatten_unique_keys(dictionary)
+                if not squash:
+                    return dictutil.normalize(dictionary)
+            return dictionary
+        return dictutil.normalize(self._params)
+
     def keys(
         self,
         flatten: bool = False,
@@ -921,9 +933,9 @@ class Smart(Generic[T]):
                         obj=default,
                         unknown_type=self.missing_value,
                     )
-                elif annotation in (dict, Dict):
+                elif annotation is dict or origin is dict:
                     representation[name] = dict()
-                elif annotation in (list, tuple, set, List, Tuple, Set):
+                elif annotation in (list, tuple, set) or origin in (list, tuple, set):
                     representation[name] = list()
                 elif annotation is not param.empty and isinstance(annotation, type):
                     if annotation in (bool, float, int, str):
