@@ -1,3 +1,4 @@
+import os.path
 from typing import Any, Dict, List, Optional, Tuple
 
 import smartparams.utils.string as strutil
@@ -7,7 +8,7 @@ def flatten_keys(
     obj: Any,
     _prefix: Optional[str] = None,
 ) -> List[str]:
-    if not obj or not isinstance(obj, dict):
+    if not obj or not isinstance(obj, dict) or not all(isinstance(key, str) for key in obj):
         return [] if _prefix is None else [_prefix]
 
     keys = []
@@ -79,27 +80,29 @@ def normalize(
     dictionary: Dict,
     trace: str = '',
 ) -> Dict:
+    if not all(isinstance(key, str) for key in dictionary):
+        return dictionary
+
     normalized_dictionary: Dict = {}
     for key, value in dictionary.items():
-        if isinstance(key, str):
-            key, separator, sub_key = key.partition(strutil.KEY_SEPARATOR)
-            if separator:
-                value = {sub_key: value}
+        key, separator, sub_key = key.partition(strutil.KEY_SEPARATOR)
+        if separator:
+            value = {sub_key: value}
 
-            current_trace = strutil.join_keys(trace, key)
+        current_trace = strutil.join_keys(trace, key)
 
-            if isinstance(value, dict):
-                value = normalize(
-                    dictionary=value,
-                    trace=current_trace,
-                )
+        if isinstance(value, dict):
+            value = normalize(
+                dictionary=value,
+                trace=current_trace,
+            )
 
-            if key in normalized_dictionary:
-                value = merge(
-                    a=normalized_dictionary[key],
-                    b=value,
-                    trace=current_trace,
-                )
+        if key in normalized_dictionary:
+            value = merge(
+                a=normalized_dictionary[key],
+                b=value,
+                trace=current_trace,
+            )
 
         normalized_dictionary[key] = value
 
@@ -116,6 +119,9 @@ def merge(
         intersection = set(a).intersection(b)
         for key in set(a).union(b):
             if key in intersection:
+                if not isinstance(key, str):
+                    raise KeyError(f"Cannot merge dictionaries with non-string keys: '{trace}'.")
+
                 dictionary[key] = merge(
                     a=a[key],
                     b=b[key],
@@ -126,4 +132,17 @@ def merge(
 
         return dictionary
 
-    raise ValueError(f"Cannot merge params with key: '{trace}'.")
+    raise ValueError(f"Cannot merge dictionaries with key: '{trace}'.")
+
+
+def flatten_unique_keys(flatten_dictionary: Dict[str, Any]) -> Dict[str, Any]:
+    keys = {key: key.split(strutil.KEY_SEPARATOR)[::-1] for key in flatten_dictionary}
+
+    for key, value in keys.items():
+        index = max(len(os.path.commonprefix((value, v))) for k, v in keys.items() if key != k)
+        keys[key] = value[: index + 1]
+
+    return {
+        strutil.KEY_SEPARATOR.join(keys[key][::-1]): value
+        for key, value in flatten_dictionary.items()
+    }
