@@ -20,12 +20,12 @@ class TestSmart(UnitCase):
     def test(self) -> None:
         smart: Smart = Smart()
 
-        self.assertIs(smart.type, dict)
+        self.assertIs(smart.type(), dict)
 
     def test__class(self) -> None:
         smart = Smart(Class)
 
-        self.assertIs(smart.type, Class)
+        self.assertIs(smart.type(), Class)
 
     def test__smart_init(self) -> None:
         smart_init = Mock()
@@ -37,12 +37,17 @@ class TestSmart(UnitCase):
     def test__function(self) -> None:
         smart = Smart(some_function)
 
-        self.assertIs(smart.type, None)
+        self.assertIs(smart.type(), None)
+
+    def test__function__no_return_type(self) -> None:
+        smart = Smart(some_function)
+
+        self.assertIs(smart.type(return_type=False), some_function)
 
     def test__lambda(self) -> None:
         smart = Smart(lambda x: x)
 
-        self.assertIs(smart.type, Any)
+        self.assertIs(smart.type(), Any)
 
     def test__not_callable(self) -> None:
         self.assertRaises(TypeError, Smart, 'not_callable')
@@ -95,12 +100,36 @@ class TestSmartDictCase(UnitCase):
     def test_dict(self) -> None:
         expected = {'arg1': 'arg1', 'arg2': ['arg2'], 'arg3': {'arg31': 'a31', 'arg32': 'a32'}}
 
-        params = self.smart.dict
+        params = self.smart.dict()
 
         self.assertEqual(expected, params)
         self.assertIsInstance(self.smart._params, dict)
         self.assertIsNot(self.smart._params, params)
 
+    def test_dict__squash(self) -> None:
+        expected = {'a.b': 1, 'a.c': 2, 'b': 3, 'd': {1: 'x'}}
+        smart: Smart = Smart(a={'b': 1, 'c': 2}, b=3, d={1: 'x'})
+
+        actual = smart.dict(squash=True)
+
+        self.assertEqual(expected, actual)
+
+    def test_dict__minimal(self) -> None:
+        expected = {'a': {'b': 1}, 'c': 2, 'b': 3, 'd': {1: 'x'}}
+        smart: Smart = Smart(a={'b': 1, 'c': 2}, b=3, d={1: 'x'})
+
+        actual = smart.dict(minimal=True)
+
+        self.assertEqual(expected, actual)
+
+    def test_dict__squash_minimal(self) -> None:
+        expected = {'a.b': 1, 'c': 2, 'b': 3, 'd': {1: 'x'}}
+        smart: Smart = Smart(a={'b': 1, 'c': 2}, b=3, d={1: 'x'})
+
+        actual = smart.dict(squash=True, minimal=True)
+
+        self.assertEqual(expected, actual)
+
     def test_keys(self) -> None:
         keys = self.smart.keys()
 
@@ -145,14 +174,14 @@ class TestSmartDictCase(UnitCase):
         value = self.smart.set('arg3.arg31', new_value)
 
         self.assertEqual('argument31', value)
-        self.assertEqual('argument31', self.smart.dict['arg3']['arg31'])
-        self.assertEqual('a32', self.smart.dict['arg3']['arg32'])
+        self.assertEqual('argument31', self.smart.dict()['arg3']['arg31'])
+        self.assertEqual('a32', self.smart.dict()['arg3']['arg32'])
 
     def test_pop(self) -> None:
         value = self.smart.pop('arg3.arg31')
 
         self.assertEqual('a31', value)
-        self.assertFalse('arg3' in self.smart.dict['arg3'])
+        self.assertFalse('arg3' in self.smart.dict()['arg3'])
 
     def test_map(self) -> None:
         function = Mock(return_value='argument31')
@@ -160,8 +189,8 @@ class TestSmartDictCase(UnitCase):
         value = self.smart.map('arg3.arg31', function)
 
         self.assertEqual('argument31', value)
-        self.assertEqual('argument31', self.smart.dict['arg3']['arg31'])
-        self.assertEqual('a32', self.smart.dict['arg3']['arg32'])
+        self.assertEqual('argument31', self.smart.dict()['arg3']['arg31'])
+        self.assertEqual('a32', self.smart.dict()['arg3']['arg32'])
 
     def test_remove(self) -> None:
         actual = self.smart.remove('arg2', 'arg3.arg31')
@@ -201,13 +230,13 @@ class TestSmartDictCase(UnitCase):
             with self.subTest(msg=msg):
                 smart.update_from(source)
 
-                self.assertTrue('arg1' in smart.dict)
-                self.assertTrue('arg2' in smart.dict)
-                self.assertTrue('arg3' in smart.dict)
-                self.assertEqual({'nested1': 'argument1'}, smart.dict['arg1'])
-                self.assertListEqual(['arg2'], smart.dict['arg2'])
-                self.assertEqual('argument31', smart.dict['arg3']['arg31'])
-                self.assertEqual('a32', smart.dict['arg3']['arg32'])
+                self.assertTrue('arg1' in smart.dict())
+                self.assertTrue('arg2' in smart.dict())
+                self.assertTrue('arg3' in smart.dict())
+                self.assertEqual({'nested1': 'argument1'}, smart.dict()['arg1'])
+                self.assertListEqual(['arg2'], smart.dict()['arg2'])
+                self.assertEqual('argument31', smart.dict()['arg3']['arg31'])
+                self.assertEqual('a32', smart.dict()['arg3']['arg32'])
 
     @patch('smartparams.smart.load_data')
     def test_update__not_override(self, load_data: Mock) -> None:
@@ -226,46 +255,46 @@ class TestSmartDictCase(UnitCase):
             with self.subTest(msg=msg):
                 smart.update_from(source, override=False)
 
-                self.assertTrue('arg1' in smart.dict)
-                self.assertTrue('arg2' in smart.dict)
-                self.assertTrue('arg3' in smart.dict)
-                self.assertEqual('arg1', smart.dict['arg1'])
-                self.assertListEqual(['arg2'], smart.dict['arg2'])
-                self.assertEqual('a31', smart.dict['arg3']['arg31'])
-                self.assertEqual('a32', smart.dict['arg3']['arg32'])
-                self.assertEqual({'a4': 'aa4'}, smart.dict['arg4'])
+                self.assertTrue('arg1' in smart.dict())
+                self.assertTrue('arg2' in smart.dict())
+                self.assertTrue('arg3' in smart.dict())
+                self.assertEqual('arg1', smart.dict()['arg1'])
+                self.assertListEqual(['arg2'], smart.dict()['arg2'])
+                self.assertEqual('a31', smart.dict()['arg3']['arg31'])
+                self.assertEqual('a32', smart.dict()['arg3']['arg32'])
+                self.assertEqual({'a4': 'aa4'}, smart.dict()['arg4'])
 
     def test_update__not_required(self) -> None:
         dictionary = {'arg1': {'nested1': 'aa1'}}
-        expected = deepcopy(self.smart.dict)
+        expected = deepcopy(self.smart.dict())
 
         self.smart.update_from(dictionary, source_key='arg2', required=False)
 
-        self.assertEqual(expected, self.smart.dict)
+        self.assertEqual(expected, self.smart.dict())
 
     def test_update__with_source_key(self) -> None:
         dictionary = {'arg': {'nested': ['arg3.arg31=argument31', 'arg1={"nested1": "argument1"}']}}
 
         self.smart.update_from(dictionary, source_key='arg.nested')
 
-        self.assertTrue('arg1' in self.smart.dict)
-        self.assertTrue('arg2' in self.smart.dict)
-        self.assertTrue('arg3' in self.smart.dict)
-        self.assertEqual({'nested1': 'argument1'}, self.smart.dict['arg1'])
-        self.assertListEqual(['arg2'], self.smart.dict['arg2'])
-        self.assertEqual('argument31', self.smart.dict['arg3']['arg31'])
-        self.assertEqual('a32', self.smart.dict['arg3']['arg32'])
+        self.assertTrue('arg1' in self.smart.dict())
+        self.assertTrue('arg2' in self.smart.dict())
+        self.assertTrue('arg3' in self.smart.dict())
+        self.assertEqual({'nested1': 'argument1'}, self.smart.dict()['arg1'])
+        self.assertListEqual(['arg2'], self.smart.dict()['arg2'])
+        self.assertEqual('argument31', self.smart.dict()['arg3']['arg31'])
+        self.assertEqual('a32', self.smart.dict()['arg3']['arg32'])
 
     def test_update__with_target_key(self) -> None:
         dictionary = {'arg31': 'argument31'}
 
         self.smart.update_from(dictionary, target_key='arg3')
 
-        self.assertTrue('arg1' in self.smart.dict)
-        self.assertTrue('arg2' in self.smart.dict)
-        self.assertTrue('arg3' in self.smart.dict)
-        self.assertEqual('argument31', self.smart.dict['arg3']['arg31'])
-        self.assertEqual('a32', self.smart.dict['arg3']['arg32'])
+        self.assertTrue('arg1' in self.smart.dict())
+        self.assertTrue('arg2' in self.smart.dict())
+        self.assertTrue('arg3' in self.smart.dict())
+        self.assertEqual('argument31', self.smart.dict()['arg3']['arg31'])
+        self.assertEqual('a32', self.smart.dict()['arg3']['arg32'])
 
     def test_update__with_key_error(self) -> None:
         dictionary = {'arg': {'nested': ['arg3.arg31 argument31']}}
@@ -344,14 +373,14 @@ class TestSmartInitCase(UnitCase):
 
         self.assertIs(self.smart.get('object'), obj)
         self.assertIsInstance(self.smart.get('object'), Class)
-        self.assertTrue('object' in self.smart.dict)
+        self.assertTrue('object' in self.smart.dict())
 
     def test_init__not_persist(self) -> None:
         obj = self.smart.init('object', persist=False)
 
         self.assertIsNot(self.smart.get('object'), obj)
         self.assertIsInstance(self.smart.get('object'), dict)
-        self.assertTrue('object' in self.smart.dict)
+        self.assertTrue('object' in self.smart.dict())
 
     def test_init__any(self) -> None:
         result = self.smart.init('value')
@@ -362,13 +391,13 @@ class TestSmartInitCase(UnitCase):
         result = self.smart.init('smart_dict')
 
         self.assertIsInstance(result, Smart)
-        self.assertIs(result.type, dict)
+        self.assertIs(result.type(), dict)
 
     def test_init__smart(self) -> None:
         result = self.smart.init('smart')
 
         self.assertIsInstance(result, Smart)
-        self.assertIs(result.type, Class)
+        self.assertIs(result.type(), Class)
 
     def test_init__type(self) -> None:
         result = self.smart.init('type')
