@@ -78,8 +78,8 @@ class TestFindNested(UnitCase):
 
 class TestNormalize(UnitCase):
     def test(self) -> None:
-        data = {'a.b': 10, 'a.c': 15, 'd': 20, 1: {'a.b.c': 10}}
-        expected = {'a': {'b': 10, 'c': 15}, 'd': 20, 1: {'a.b.c': 10}}
+        data = {'a.b': 10, 'a.c': 15, 'd': 20, 'e': {1: {'a.b.c': 10}}}
+        expected = {'a': {'b': 10, 'c': 15}, 'd': 20, 'e': {1: {'a.b.c': 10}}}
 
         actual = dictutil.normalize(data)
 
@@ -128,3 +128,26 @@ class TestNormalize(UnitCase):
         data = {'a.b': 15, 'a.b.c.d': 10}
 
         self.assertRaises(ValueError, dictutil.normalize, data)
+
+
+class TestMerge(UnitCase):
+    def test(self) -> None:
+        a = {'a': {'b': 10}, 'd': 20}
+        b = {'a': {'c': 15}}
+        expected = {'a': {'b': 10, 'c': 15}, 'd': 20}
+
+        actual = dictutil.merge(a, b)
+
+        self.assertEqual(expected, actual)
+
+    def test__error__overridden(self) -> None:
+        a = {'a': {'b': 10}, 'd': 20}
+        b = {'a': {'c': 15}, 'd': 20}
+
+        self.assertRaises(ValueError, dictutil.merge, a, b)
+
+    def test__error__non_string(self) -> None:
+        a = {'a': {'b': 10}, 1: {'d': 11}}
+        b = {'a': {'c': 15}, 1: {'e': 22}}
+
+        self.assertRaises(KeyError, dictutil.merge, a, b)
